!-------------------------------------------------------------------------
! f90gl Version 1.0 September 1996
! 
! William F. Mitchell
! Applied and Computational Mathematics Division
! National Institute of Standards and Technology
! Gaithersburg, MD 20899
! william.mitchell@nist.gov
! 
! This fortran interface for GLUT software and documentation have been produced
! as part of work done by the U.S. Government, and are not subject to copyright
! in the United States.
! 
! The mention of specific products, trademarks, or brand names in the
! documentation is for purposes of identification only.  Such mention is not to
! be interpreted in any way as an endorsement or certification of such products
! or brands by the National Institute of Standards and Technology.
! All trademarks mentioned herein belong to their respective owners.
!-------------------------------------------------------------------------

module dino_mod

use f90gl
use f90glu
use f90glut

implicit none

integer, parameter :: &
  RESERVED   =  0, &
  BODY_SIDE  =  1, &
  BODY_EDGE  =  2, &
  BODY_WHOLE =  3, &
  ARM_SIDE   =  4, &
  ARM_EDGE   =  5, &
  ARM_WHOLE  =  6, &
  LEG_SIDE   =  7, &
  LEG_EDGE   =  8, &
  LEG_WHOLE  =  9, &
  EYE_SIDE   = 10, &
  EYE_EDGE   = 11, &
  EYE_WHOLE  = 12, &
  DINOSAUR   = 13

type coord
   real(f90glfloat) :: x,y
end type coord

type(coord) :: angle = coord(-150.,0.) ! in degrees
type(coord) :: begin
logical :: moving
integer :: W = 300, H = 300
real(f90glfloat) :: bodyWidth = 2.0
logical :: newModel = .true.
real(f90glfloat) :: lightZeroPosition(4) = (/10.0, 4.0, 10.0, 1.0/), &
        lightZeroColor(4) = (/0.8, 1.0, 0.8, 1.0/), & ! green-tinted
        lightOnePosition(4) = (/-1.0, -2.0, 1.0, 0.0/), &
        lightOneColor(4) = (/0.6, 0.3, 0.2, 1.0/), & ! red-tinted
        skinColor(4) = (/0.1, 1.0, 0.1, 1.0/), &
        eyeColor(4) = (/1.0, 0.2, 0.2, 1.0/)

logical :: lightZeroSwitch = .true., lightOneSwitch = .true.

contains

subroutine extrudeSolidFromPolygon(data, dataSize, &
                                   thickness, side, edge, whole)
real(f90glfloat) :: data(:,:)
integer, intent(in) :: dataSize
real(f90glfloat), intent(in) :: thickness
integer(f90gluint), intent(in) :: side, edge, whole

  type(f90GLUtriangulatorObj), save, pointer :: tobj
  logical :: tobj_null = .true.

  
  real(f90gldouble) :: vertex(3)
  real(f90glfloat) dx, dy, len
  integer i, count
  external fglBegin, fglVertex2fv, fglEnd

  count = dataSize/2

  if (tobj_null) then
    tobj => f90gluNewTess()  ! create and initialize a GLU
                             ! polygon * * tesselation object
    call f90gluTessCallback(tobj, GLU_BEGIN, fglBegin)
    call f90gluTessCallback(tobj, GLU_VERTEX, fglVertex2fv)  ! semi-tricky 
    call f90gluTessCallback(tobj, GLU_END, fglEnd)
  endif
  call f90glNewList(side, GL_COMPILE)
  call f90glShadeModel(GL_SMOOTH)  ! smooth minimizes seeing tessellation
  call f90gluBeginPolygon(tobj)
  do i=1,count
! would like to use an array constructor as`
!    vertex = (/data(1,i), data(2,i), 0./)
! but NAG f90 on linux crashes here
     vertex(1) = data(1,i)
     vertex(2) = data(2,i)
     vertex(3) = 0.
! would like to pass data(:,i) but some compilers will copy array sections
! to a temp instead of passing the address, so the C pointer assignment to
! data is invalid
    call f90gluTessVertex(tobj, vertex, data(1,i))
  end do
  call f90gluEndPolygon(tobj)
  call f90glEndList()
  call f90glNewList(edge, GL_COMPILE)
  call f90glShadeModel(GL_FLAT)  ! flat shade keeps angular hands
                            ! from being * * "smoothed"
  call f90glBegin(GL_QUAD_STRIP)
  do i=0,count
!      mod function handles closing the edge
    call f90glVertex3f(data(1,mod(i,count)+1), data(2,mod(i,count)+1), 0.0)
    call f90glVertex3f(data(1,mod(i,count)+1), data(2,mod(i,count)+1), thickness)
!       Calculate a unit normal by dividing by Euclidean
!       distance. We * could be lazy and use
!       glEnable(GL_NORMALIZE) so we could pass in * arbitrary
!       normals for a very slight performance hit.
    dx = data(2,mod((i + 1),count)+1) - data(2,mod(i,count)+1)
    dy = data(1,mod(i,count)+1) - data(1,mod((i + 1), count)+1)
    len = sqrt(dx * dx + dy * dy)
    call f90glNormal3f(dx / len, dy / len, 0.0)
  end do
  call f90glEnd()
  call f90glEndList()
  call f90glNewList(whole, GL_COMPILE)
  call f90glFrontFace(GL_CW)
  call f90glCallList(edge)
  call f90glNormal3f(0.0, 0.0, -1.0)  ! constant normal for side
  call f90glCallList(side)
  call f90glPushMatrix()
  call f90glTranslatef(0.0, 0.0, thickness)
  call f90glFrontFace(GL_CCW)
  call f90glNormal3f(0.0, 0.0, 1.0)  ! opposite normal for other side
  call f90glCallList(side)
  call f90glPopMatrix()
  call f90glEndList()
return
end subroutine extrudeSolidFromPolygon

subroutine makeDinosaur()

  real(f90glfloat) :: bodyWidth = 3.0
real(f90glfloat) :: body(2,22) = reshape( &
  (/ 0., 3., 1., 1., 5., 1., 8., 4., 10., 4., 11., 5., &
  11., 11.5, 13., 12., 13., 13., 10., 13.5, 13., 14., 13., 15., 11., 16., &
  8., 16., 7., 15., 7., 13., 8., 12., 7., 11., 6., 6., 4., 3., 3., 2., 1., 2. /), &
  (/2, 22/) )
real(f90glfloat) :: arm(2,16) = reshape( &
  (/ 8., 10., 9., 9., 10., 9., 13., 8., 14., 9., 16., 9., &
  15., 9.5, 16., 10., 15., 10., 15.5, 11., 14.5, 10., 14., 11., 14., 10., &
  13., 9., 11., 11., 9., 11. /), &
  (/2, 16/) )
real(f90glfloat) :: leg(2,14) = reshape( &
  (/ 8., 6., 8., 4., 9., 3., 9., 2., 8., 1., 8., 0.5, 9., 0., &
  12., 0., 10., 1., 10., 2., 12., 4., 11., 6., 10., 7., 9., 7. /), &
  (/2, 14/) )
real(f90glfloat) :: eye(2,6) = reshape( &
  (/ 8.75, 15., 9., 14.7, 9.6, 14.7, 10.1, 15., 9.6, 15.25, 9., 15.25 /), &
  (/2, 6/) )

  call extrudeSolidFromPolygon(body, size(body), bodyWidth, &
    BODY_SIDE, BODY_EDGE, BODY_WHOLE)
  call extrudeSolidFromPolygon(arm, size(arm), bodyWidth / 4, &
    ARM_SIDE, ARM_EDGE, ARM_WHOLE)
  call extrudeSolidFromPolygon(leg, size(leg), bodyWidth / 2, &
    LEG_SIDE, LEG_EDGE, LEG_WHOLE)
  call extrudeSolidFromPolygon(eye, size(eye), bodyWidth + 0.2, &
    EYE_SIDE, EYE_EDGE, EYE_WHOLE)
  call f90glNewList(DINOSAUR, GL_COMPILE)
  call f90glMaterialfv(GL_FRONT, GL_DIFFUSE, skinColor)
  call f90glCallList(BODY_WHOLE)
  call f90glPushMatrix()
  call f90glTranslatef(0.0, 0.0, bodyWidth)
  call f90glCallList(ARM_WHOLE)
  call f90glCallList(LEG_WHOLE)
  call f90glTranslatef(0.0, 0.0, -bodyWidth - bodyWidth / 4)
  call f90glCallList(ARM_WHOLE)
  call f90glTranslatef(0.0, 0.0, -bodyWidth / 4)
  call f90glCallList(LEG_WHOLE)
  call f90glTranslatef(0.0, 0.0, bodyWidth / 2 - 0.1)
  call f90glMaterialfv(GL_FRONT, GL_DIFFUSE, eyeColor)
  call f90glCallList(EYE_WHOLE)
  call f90glPopMatrix()
  call f90glEndList()
end subroutine makedinosaur

subroutine recalcModelView()

real, parameter :: pi = 3.1415926
  call f90glPopMatrix()
  call f90glPushMatrix()
  call f90glRotatef(angle%x, 0.0, 1.0, 0.0)
  call f90glRotatef(angle%y, cos(pi*angle%x/180.), 0.0, sin(pi*angle%x/180.))
  call f90glTranslatef(-8., -8., -bodyWidth / 2)
  newModel = .false.
end subroutine recalcModelView

end module dino_mod

subroutine redraw()
use dino_mod

  if (newModel) then
    call recalcModelView()
  endif
  call f90glClear(ior(GL_COLOR_BUFFER_BIT,GL_DEPTH_BUFFER_BIT))
  call f90glCallList(DINOSAUR)
  call f90glutSwapBuffers()
end subroutine redraw

subroutine mouse(button, state, x, y)
use dino_mod
integer(f90glcint) :: button, state, x, y

  if (button == GLUT_LEFT_BUTTON .and. state == GLUT_DOWN) then
    moving = .true.
    begin = coord(x,y)
  endif
  if (button == GLUT_LEFT_BUTTON .and. state == GLUT_UP) then
    moving = .false.
  endif
end subroutine mouse

subroutine motion(x, y)
use dino_mod
integer(f90glcint) :: x, y

  if (moving) then
    angle%x = angle%x + (x - begin%x)
    angle%y = angle%y + (y - begin%y)
    begin = coord(x,y)
    newModel = .true.
    call f90glutPostRedisplay()
  endif
end subroutine motion

subroutine controlLights(value)
use dino_mod
integer(f90glcint) value

  select case (value)
  case (1)
    lightZeroSwitch = .not. lightZeroSwitch
    if (lightZeroSwitch) then
      call f90glEnable(GL_LIGHT0)
    else
      call f90glDisable(GL_LIGHT0)
    endif
  case (2)
    lightOneSwitch = .not. lightOneSwitch
    if (lightOneSwitch) then
      call f90glEnable(GL_LIGHT1)
    else
      call f90glDisable(GL_LIGHT1)
    endif
  end select
  call f90glutPostRedisplay()
end subroutine controlLights

program main

use dino_mod
external redraw, mouse, motion, controlLights

  call f90glutInit()
  call f90glutInitDisplayMode(ior(ior(GLUT_RGB,GLUT_DOUBLE),GLUT_DEPTH))
  i = f90glutCreateWindow("glutdino")
  call f90glutDisplayFunc(redraw)
  call f90glutMouseFunc(mouse)
  call f90glutMotionFunc(motion)
  i = f90glutCreateMenu(controlLights)
  call f90glutAddMenuEntry("Toggle right light", 1)
  call f90glutAddMenuEntry("Toggle left light", 2)
  call f90glutAttachMenu(GLUT_RIGHT_BUTTON)
  call makeDinosaur()
  call f90glEnable(GL_CULL_FACE)
  call f90glEnable(GL_DEPTH_TEST)
  call f90glEnable(GL_LIGHTING)
  call f90glMatrixMode(GL_PROJECTION)
  call f90gluPerspective(40.0_f90gldouble, & ! field of view in degree
                          1.0_f90gldouble, & ! aspect ratio
                          1.0_f90gldouble, & ! Z near
                         40.0_f90gldouble)   ! Z far
  call f90glMatrixMode(GL_MODELVIEW)
  call f90gluLookAt( &
   0.0_f90gldouble, 0.0_f90gldouble, 30.0_f90gldouble, & ! eye is at (0,0,30)
   0.0_f90gldouble, 0.0_f90gldouble, 0.0_f90gldouble, & ! center is at (0,0,0)
   0.0_f90gldouble, 1.0_f90gldouble, 0._f90gldouble) ! up is in postivie Y direction
  call f90glPushMatrix()  ! dummy push so we can pop on model recalc 
  call f90glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1)
  call f90glLightfv(GL_LIGHT0, GL_POSITION, lightZeroPosition)
  call f90glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor)
  call f90glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.1)
  call f90glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.05)
  call f90glLightfv(GL_LIGHT1, GL_POSITION, lightOnePosition)
  call f90glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOneColor)
  call f90glEnable(GL_LIGHT0)
  call f90glEnable(GL_LIGHT1)
  call f90glutMainLoop()
end program main
