!-------------------------------------------------------------------------
! f90gl Version 1.0 September 1996
! 
! William F. Mitchell
! Applied and Computational Mathematics Division
! National Institute of Standards and Technology
! Gaithersburg, MD 20899
! william.mitchell@nist.gov
! 
! This fortran interface for GLUT software and documentation have been produced
! as part of work done by the U.S. Government, and are not subject to copyright
! in the United States.
! 
! The mention of specific products, trademarks, or brand names in the
! documentation is for purposes of identification only.  Such mention is not to
! be interpreted in any way as an endorsement or certification of such products
! or brands by the National Institute of Standards and Technology.
! All trademarks mentioned herein belong to their respective owners.
!-------------------------------------------------------------------------

module olympic_mod
use f90gl
use f90glu
use f90glut

integer, parameter :: &
   XSIZE      = 100, &
   YSIZE      = 75, &
   RINGS      = 5, &
   BLUERING   = 0, &
   BLACKRING  = 1, &
   REDRING    = 2, &
   YELLOWRING = 3, &
   GREENRING  = 4, &
   BLACK      = 0, &
   RED        = 1, &
   GREEN      = 2, &
   YELLOW     = 3, &
   BLUE       = 4, &
   MAGENTA    = 5, &
   CYAN       = 6, &
   WHITE      = 7

real(f90glfloat), parameter :: BACKGROUND = 8.

integer, parameter :: double = kind(0.0d0)

real, parameter :: M_PI = 3.141592654

integer(f90glenum) rgb, doubleBuffer, directRender

integer(f90glubyte) rgb_colors(0:RINGS-1,0:2)
integer(f90glint) mapped_colors(0:RINGS-1)
real(f90glfloat) dests(0:RINGS-1,0:2)
real(f90glfloat) offsets(0:RINGS-1,0:2)
real(f90glfloat) angs(0:RINGS-1)
real(f90glfloat) rotAxis(0:RINGS-1,0:2)
integer iters(0:RINGS-1)
integer(f90gluint) theTorus

contains

subroutine FillTorus(rc, numc, rt, numt)
real, intent(in) :: rc, rt
integer, intent(in) :: numc, numt

  integer :: i, j, k
  real :: s, t
  real(f90glfloat) x, y, z
  real pi, twopi

  pi = M_PI
  twopi = 2 * pi

  do i = 0, numc-1
    call f90glBegin(GL_QUAD_STRIP)
    do j = 0, numt
      do k = 1, 0, -1
        s = mod((i + k), numc) + 0.5
        t = mod(j, numt)

        x = cos(t * twopi / numt) * cos(s * twopi / numc)
        y = sin(t * twopi / numt) * cos(s * twopi / numc)
        z = sin(s * twopi / numc)
        call f90glNormal3f(x, y, z)

        x = (rt + rc * cos(s * twopi / numc)) * cos(t * twopi / numt)
        y = (rt + rc * cos(s * twopi / numc)) * sin(t * twopi / numt)
        z = rc * sin(s * twopi / numc)
        call f90glVertex3f(x, y, z)
      end do
    end do
    call f90glEnd()
  end do
return
end subroutine filltorus

function Clamp(iters_left,t)
real :: clamp
integer, intent(in) :: iters_left
real, intent(in) :: t

  if (iters_left < 3) then
    clamp = 0.0
  else
    clamp = (iters_left - 2) * t / iters_left
  endif
return
end function clamp

function MyRand()
real :: myrand
real :: rval

  call random_number(rval)
  myrand = 10.0 * (rval - 0.5)

return
end function myrand

subroutine ReInit()

  integer :: i
  real :: deviation

  deviation = MyRand() / 2
  deviation = deviation * deviation
  do i = 0, RINGS-1
    offsets(i,0) = MyRand()
    offsets(i,1) = MyRand()
    offsets(i,2) = MyRand()
    angs(i) = 260.0 * MyRand()
    rotAxis(i,0) = MyRand()
    rotAxis(i,1) = MyRand()
    rotAxis(i,2) = MyRand()
    iters(i) = (deviation * MyRand() + 60.0)
  end do

return
end subroutine reinit

subroutine Init()

  real(f90glfloat) :: top_y = 1.0
  real(f90glfloat) :: bottom_y = 0.0
  real(f90glfloat) :: top_z = 0.15
  real(f90glfloat) :: bottom_z = 0.69
  real(f90glfloat) :: spacing = 2.5
  real(f90glfloat), save :: lmodel_ambient(4) = (/0.0, 0.0, 0.0, 0.0/)
  real(f90glfloat), save :: lmodel_twoside(1) = (/GL_FALSE/)
  real(f90glfloat), save :: lmodel_local(1) = (/GL_FALSE/)
  real(f90glfloat), save :: light0_ambient(4) = (/0.1, 0.1, 0.1, 1.0/)
  real(f90glfloat), save :: light0_diffuse(4) = (/1.0, 1.0, 1.0, 0.0/)
  real(f90glfloat), save :: light0_position(4) = (/0.8660254, 0.5, 1.0, 0.0/)
  real(f90glfloat), save :: light0_specular(4) = (/1.0, 1.0, 1.0, 0.0/)
  real(f90glfloat), save :: bevel_mat_ambient(4) = (/0.0, 0.0, 0.0, 1.0/)
  real(f90glfloat), save :: bevel_mat_shininess(1) = (/40.0/)
  real(f90glfloat), save :: bevel_mat_specular(4) = (/1.0, 1.0, 1.0, 0.0/)
  real(f90glfloat), save :: bevel_mat_diffuse(4) = (/1.0, 0.0, 0.0, 0.0/)

  call random_seed()
  call ReInit()
  rgb_colors = 0
  rgb_colors(BLUERING,2) = 255
  rgb_colors(REDRING,0) = 255
  rgb_colors(GREENRING,1) = 255
  rgb_colors(YELLOWRING,0) = 255
  rgb_colors(YELLOWRING,1) = 255
  mapped_colors(BLUERING) = BLUE
  mapped_colors(REDRING) = RED
  mapped_colors(GREENRING) = GREEN
  mapped_colors(YELLOWRING) = YELLOW
  mapped_colors(BLACKRING) = BLACK

  dests(BLUERING,:) = (/-spacing, top_y, top_z/)
  dests(BLACKRING,:) = (/0.0, top_y, top_z/)
  dests(REDRING,:) = (/spacing, top_y, top_z/)
  dests(YELLOWRING,:) = (/-spacing / 2.0, bottom_y, bottom_z/)
  dests(GREENRING,:) = (/spacing / 2.0, bottom_y, bottom_z/)

  theTorus = f90glGenLists(1)
  call f90glNewList(theTorus, GL_COMPILE)
  call FillTorus(0.1, 8, 1.0, 25)
  call f90glEndList()

  call f90glEnable(GL_CULL_FACE)
  call f90glCullFace(GL_BACK)
  call f90glEnable(GL_DEPTH_TEST)
  call f90glClearDepth(1.0_f90glclampd)

  if (rgb == GL_TRUE) then
    call f90glClearColor(0.5_f90glclampf, 0.5_f90glclampf, 0.5_f90glclampf, 0.0_f90glclampf)
    call f90glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient)
    call f90glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse)
    call f90glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular)
    call f90glLightfv(GL_LIGHT0, GL_POSITION, light0_position)
    call f90glEnable(GL_LIGHT0)

    call f90glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_local)
    call f90glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside)
    call f90glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient)
    call f90glEnable(GL_LIGHTING)

    call f90glMaterialfv(GL_FRONT, GL_AMBIENT, bevel_mat_ambient)
    call f90glMaterialfv(GL_FRONT, GL_SHININESS, bevel_mat_shininess)
    call f90glMaterialfv(GL_FRONT, GL_SPECULAR, bevel_mat_specular)
    call f90glMaterialfv(GL_FRONT, GL_DIFFUSE, bevel_mat_diffuse)

    call f90glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE)
    call f90glEnable(GL_COLOR_MATERIAL)
    call f90glShadeModel(GL_SMOOTH)
  else
    call f90glClearIndex(BACKGROUND)
    call f90glShadeModel(GL_FLAT)
  endif

  call f90glMatrixMode(GL_PROJECTION)
  call f90gluPerspective(45._f90gldouble, 1.33_f90gldouble, 0.1_f90gldouble, 100.0_f90gldouble)
  call f90glMatrixMode(GL_MODELVIEW)

return
end subroutine init

end module olympic_mod

subroutine Idle()
use olympic_mod

  integer :: i, j
  integer(f90glenum) :: more = GL_FALSE
  external glutnull

  do i = 0, RINGS-1
    if (iters(i) /= 0) then
      do j = 0, 2
        offsets(i,j) = Clamp(iters(i), offsets(i,j))
      end do
      angs(i) = Clamp(iters(i), angs(i))
      iters(i) = iters(i) - 1
      more = GL_TRUE
    end if
  end do
  if (more == GL_TRUE) then
    call f90glutPostRedisplay()
  else
    call f90glutIdleFunc(glutnull)
  endif

return
end subroutine idle

subroutine Reshape(width,height)
use olympic_mod
integer(f90glcint) width, height

! if f90glcint is not the same as f90glsizei, width and height will
! need to be copied to variables of the later kind

  call f90glViewport(0_f90glint, 0_f90glint, width, height)

return
end subroutine reshape

subroutine Key(ikey, x, y)
use olympic_mod
integer(f90glcint) ikey, x, y
external idle

  select case(ikey)
  case (27) ! esc
    stop
  case (iachar(' '))
    call ReInit()
    call f90glutIdleFunc(Idle)
  end select

return
end subroutine key

! fortran handling of command line arguments is nonstandard, so
! this feature is omitted.  Here is the original C code.

!GLenum
!Args(int argc, char **argv)
!{
!  GLint i;
!
!  rgb = GL_TRUE;
!  doubleBuffer = GL_TRUE;
!
!  for (i = 1; i < argc; i++) {
!    if (strcmp(argv[i], "-ci") == 0) {
!      rgb = GL_FALSE;
!    } else if (strcmp(argv[i], "-rgb") == 0) {
!      rgb = GL_TRUE;
!    } else if (strcmp(argv[i], "-sb") == 0) {
!      doubleBuffer = GL_FALSE;
!    } else if (strcmp(argv[i], "-db") == 0) {
!      doubleBuffer = GL_TRUE;
!    } else {
!      printf("%s (Bad option).\n", argv[i]);
!      return GL_FALSE;
!    }
!  }
!  return GL_TRUE;
!}

subroutine visible(vis)
use olympic_mod
integer(f90glcint) vis
external glutnull, idle

  if (vis == GLUT_VISIBLE) then
    call f90glutIdleFunc(Idle)
  else
    call f90glutIdleFunc(glutnull)
  endif

return
end subroutine visible

subroutine DrawScene()
use olympic_mod

  integer :: i

  call f90glPushMatrix()

  call f90glClear(ior(GL_COLOR_BUFFER_BIT,GL_DEPTH_BUFFER_BIT))
  call f90gluLookAt(0._f90gldouble, 0._f90gldouble, 10._f90gldouble, &
                    0._f90gldouble, 0._f90gldouble, 0._f90gldouble, &
                    0._f90gldouble, 1._f90gldouble, 0._f90gldouble)

  do i = 0, RINGS-1
    if (rgb == GL_TRUE) then
      call f90glColor3ubv(rgb_colors(i,:))
    else
      call f90glIndexi(mapped_colors(i))
    endif
    call f90glPushMatrix()
    call f90glTranslatef(dests(i,0) + offsets(i,0), dests(i,1) + offsets(i,1), &
      dests(i,2) + offsets(i,2))
    call f90glRotatef(angs(i), rotAxis(i,0), rotAxis(i,1), rotAxis(i,2))
    call f90glCallList(theTorus)
    call f90glPopMatrix()
  end do

  call f90glPopMatrix()
  if (doubleBuffer == GL_TRUE) then
    call f90glutSwapBuffers()
  else
    call f90glFlush()
  endif
return
end subroutine drawscene

program main

  use olympic_mod

  integer(f90glenum) :: type
  integer :: i
  external reshape, key, visible, drawscene

  call f90glutInitWindowSize(400_f90glcint, 300_f90glcint)

! not checking command line arguments
!  glutInit(&argc, argv);
!  if (Args(argc, argv) == GL_FALSE) {
!    exit(1);
!  }
  call f90glutinit()
  rgb = GL_TRUE           ! default values which could have been
  doubleBuffer = GL_TRUE  ! overwritten by command line arguments

  if (rgb == GL_TRUE) then
     type = GLUT_RGB
  else
     type = GLUT_INDEX
  endif

  if (doubleBuffer == GL_TRUE) then
     type = ior(type,GLUT_DOUBLE)
  else
     type = ior(type,GLUT_SINGLE)
  endif
  call f90glutInitDisplayMode(type)

  i = f90glutCreateWindow("Olympic")

  call Init()

  call f90glutReshapeFunc(Reshape)
  call f90glutKeyboardFunc(Key)
  call f90glutDisplayFunc(DrawScene)

  call f90glutVisibilityFunc(visible)

  call f90glutMainLoop()

end program main
