!-------------------------------------------------------------------------
! f90gl Version 1.0 September 1996
! 
! William F. Mitchell
! Applied and Computational Mathematics Division
! National Institute of Standards and Technology
! Gaithersburg, MD 20899
! william.mitchell@nist.gov
! 
! This fortran interface for Mesa software and documentation have been produced
! as part of work done by the U.S. Government, and are not subject to copyright
! in the United States.
! 
! The mention of specific products, trademarks, or brand names in the
! documentation is for purposes of identification only.  Such mention is not to
! be interpreted in any way as an endorsement or certification of such products
! or brands by the National Institute of Standards and Technology.
! All trademarks mentioned herein belong to their respective owners.
!-------------------------------------------------------------------------

module f90gl_kinds

! Define the types

! The f90glboolean kind number must be set by the user because fortran 90
! does not provide the same utilities for logical that it does for the
! numeric types.  If your system supports one byte logicals, then set
! f90glboolean to the kind number for one byte logicals and set
! f90glone_byte_logical to .true.  Some known systems are listed below.
! If your system does not have one byte logical or you don't know, then
! use the default kind of logical and set f90glone_byte_logical to .false.

! default kind of logical; probably not a one byte logical
integer, parameter :: f90glboolean = kind(.true.)
logical, parameter :: f90glone_byte_logical = .false.

! NAGWare f90
!integer, parameter :: f90glboolean = 1
!logical, parameter :: f90glone_byte_logical = .true.


! f90glcptr should be set to the number of bytes (technically, "character
! storage units") needed for a C pointer.
! f90glnullptr should be a character string corresponding to a C null
! pointer, i.e., a string of f90glcptr char(0)'s.

integer, parameter :: f90glcptr = 4
character(f90glcptr), parameter :: f90glnullptr = char(0)//char(0)//char(0)//char(0)

! The remaining kind numbers are set automatically, but may need to be
! modified on some systems to match the C types.

integer, parameter :: &
   f90glbyte     = selected_int_kind(2), &
   f90glubyte    = selected_int_kind(2), &
   f90glshort    = selected_int_kind(4), &
   f90glushort   = selected_int_kind(4), &
   f90glint      = selected_int_kind(9), &
   f90gluint     = selected_int_kind(9), &
   f90glenum     = selected_int_kind(9), &
   f90glbitfield = selected_int_kind(9), &
   f90glsizei    = selected_int_kind(9), &
   f90glfloat    = selected_real_kind(6), &
   f90glclampf   = selected_real_kind(6), &
   f90gldouble   = selected_real_kind(15), &
   f90glclampd   = selected_real_kind(15), &
   f90glcint     = selected_int_kind(9), &
   f90glcuint    = selected_int_kind(9), &
   f90glculong   = selected_int_kind(9), &
   f90glcfloat   = selected_real_kind(6)

! Determine if the short types exist

logical, parameter :: &
   f90glshort_is_int = (f90glshort == f90glint), &
   f90glbyte_is_int = (f90glbyte == f90glint), &
   f90glbyte_is_short = (f90glbyte == f90glshort .and. .not. f90glbyte_is_int)

end module f90gl_kinds
